package org.commonjava.depbase.web.base.rest;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.apache.maven.graph.common.DependencyScope;
import org.apache.maven.graph.effective.filter.DependencyFilter;
import org.apache.maven.graph.effective.filter.ExtensionFilter;
import org.apache.maven.graph.effective.filter.OrFilter;
import org.apache.maven.graph.effective.filter.ParentFilter;
import org.apache.maven.graph.effective.filter.PluginRuntimeFilter;
import org.apache.maven.graph.effective.filter.ProjectRelationshipFilter;
import org.commonjava.tensor.agg.AggregationOptions;
import org.commonjava.tensor.agg.DefaultAggregatorOptions;

public final class RequestUtils
{

    private RequestUtils()
    {
    }

    public static AggregationOptions createAggregationOptions( final HttpServletRequest request )
    {
        final DefaultAggregatorOptions options = new DefaultAggregatorOptions();
        options.setFilter( createFilter( request ) );

        if ( getBooleanParamWithDefault( request, "disc", false ) )
        {
            options.setDiscoveryEnabled( true );

            options.setDiscoveryTimeoutMillis( getLongParamWithDefault( request, "to",
                                                                        options.getDiscoveryTimeoutMillis() ) );

            options.setProcessIncompleteSubgraphs( getBooleanParamWithDefault( request, "inc", true ) );
            options.setProcessIncompleteSubgraphs( getBooleanParamWithDefault( request, "var", true ) );
        }

        return options;
    }

    public static ProjectRelationshipFilter createFilter( final HttpServletRequest request )
    {
        //        @QueryParam( "d" ) @DefaultValue( "true" ) final boolean dependencies,
        //        @QueryParam( "pa" ) @DefaultValue( "true" ) final boolean parents,
        //        @QueryParam( "pl" ) @DefaultValue( "false" ) final boolean plugins,
        //        @QueryParam( "e" ) @DefaultValue( "false" ) final boolean extensions,
        //        @QueryParam( "s" ) @DefaultValue( "runtime" ) final String scope 

        final List<ProjectRelationshipFilter> filters = new ArrayList<ProjectRelationshipFilter>();
        if ( getBooleanParamWithDefault( request, "d", true ) )
        {
            filters.add( new DependencyFilter( DependencyScope.getScope( getStringParamWithDefault( request, "s",
                                                                                                    "runtime" ) ) ) );
        }
        if ( getBooleanParamWithDefault( request, "pa", true ) )
        {
            filters.add( new ParentFilter() );
        }
        if ( getBooleanParamWithDefault( request, "pl", false ) )
        {
            filters.add( new PluginRuntimeFilter() );
        }
        if ( getBooleanParamWithDefault( request, "e", false ) )
        {
            filters.add( new ExtensionFilter() );
        }

        return new OrFilter( filters );
    }

    public static String getStringParamWithDefault( final HttpServletRequest request, final String key, final String def )
    {
        String value = request.getParameter( key );
        if ( value == null || value.trim()
                                   .length() < 1 )
        {
            value = def;
        }

        return value;
    }

    public static long getLongParamWithDefault( final HttpServletRequest request, final String key, final long def )
    {
        final String value = request.getParameter( key );
        if ( value == null || value.trim()
                                   .length() < 1 )
        {
            return def;
        }

        return Long.parseLong( value );
    }

    public static boolean getBooleanParamWithDefault( final HttpServletRequest request, final String key,
                                                      final boolean def )
    {
        final String value = request.getParameter( key );
        boolean val;
        if ( value == null || value.trim()
                                   .length() < 1 )
        {
            val = def;
        }
        else
        {
            val = Boolean.parseBoolean( value );
        }

        return val;
    }
}
